==> etterna-0.70.0.tar.gz: etterna-0.70.0/src/RageUtil/Graphics/RageSurface_Load.cpp <==
#include "Etterna/Globals/global.h"
#include "Etterna/Actor/Base/ActorUtil.h"
#include "RageUtil/File/RageFile.h"
#include "RageUtil/Misc/RageLog.h"
#include "RageSurface_Load.h"
#define STB_IMAGE_IMPLEMENTATION
#include <stb/stb_image.h>
#include "RageUtil/Utils/RageUtil.h"
#include "RageSurface.h"
#include <set>

RageSurfaceUtils::OpenResult
RageSurface_stb_Load(const std::string& sPath,
					 RageSurface*& ret,
					 bool bHeaderOnly,
					 std::string& error)
{
	RageFile f;
	if (!f.Open(sPath)) {
		error = f.GetError();
		return RageSurfaceUtils::OPEN_FATAL_ERROR;
	}

	int x, y, n;
	unsigned char* doot = stbi_load(f.GetPath().c_str(), &x, &y, &n, 4);
	if (doot == nullptr) {
		return RageSurfaceUtils::OPEN_FATAL_ERROR;
	}
	if (bHeaderOnly) {
		ret = CreateSurfaceFrom(x, y, 32, 0, 0, 0, 0, nullptr, x * 4);
		stbi_image_free(doot);
		return RageSurfaceUtils::OPEN_OK;
	} else {
		ret = CreateSurfaceFrom(x,
								y,
								32,
								Swap32BE(0xFF000000),
								Swap32BE(0x00FF0000),
								Swap32BE(0x0000FF00),
								Swap32BE(0x000000FF),
								doot,
								x * 4);
	}

	if (ret == nullptr) {
		stbi_image_free(doot);
		return RageSurfaceUtils::OPEN_UNKNOWN_FILE_FORMAT; // XXX
	}
	ret->stb_loadpoint = true;
	return RageSurfaceUtils::OPEN_OK;
}
static RageSurface*
TryOpenFile(const std::string& sPath,
			bool bHeaderOnly,
			std::string& error,
			const std::string& format,
			bool& bKeepTrying)
{
	RageSurface* ret = nullptr;
	RageSurfaceUtils::OpenResult result;
	result = RageSurface_stb_Load(sPath, ret, bHeaderOnly, error);

	if (result == RageSurfaceUtils::OPEN_OK) {
		ASSERT(ret != nullptr);
		return ret;
	}

	LOG->Trace("Format %s failed: %s", format.c_str(), error.c_str());
	return nullptr;
}

RageSurface*
RageSurfaceUtils::LoadFile(const std::string& sPath,
						   std::string& error,
						   bool bHeaderOnly)
{
	{
		RageFile TestOpen;
		if (!TestOpen.Open(sPath)) {
			error = TestOpen.GetError();
			return nullptr;
		}
	}

	set<std::string> FileTypes;
	vector<std::string> const& exts =
	  ActorUtil::GetTypeExtensionList(FT_Bitmap);
	for (vector<std::string>::const_iterator curr = exts.begin();
		 curr != exts.end();
		 ++curr) {
		FileTypes.insert(*curr);
	}

	std::string format = GetExtension(sPath);
	MakeLower(format);

	bool bKeepTrying = true;

	/* If the extension matches a format, try that first. */
	if (FileTypes.find(format) != FileTypes.end()) {
		RageSurface* ret =
		  TryOpenFile(sPath, bHeaderOnly, error, format, bKeepTrying);
		if (ret)
			return ret;
		FileTypes.erase(format);
	}

	for (set<std::string>::iterator it = FileTypes.begin();
		 bKeepTrying && it != FileTypes.end();
		 ++it) {
		RageSurface* ret =
		  TryOpenFile(sPath, bHeaderOnly, error, *it, bKeepTrying);
		if (ret) {
			LOG->UserLog("Graphic file", sPath, "is really %s", it->c_str());
			return ret;
		}
	}

	return nullptr;
}

==> etterna-0.70.1.tar.gz: etterna-0.70.1/src/RageUtil/Graphics/RageSurface_Load.cpp <==
#include "Etterna/Globals/global.h"
#include "Etterna/Actor/Base/ActorUtil.h"
#include "RageUtil/File/RageFile.h"
#include "RageUtil/Misc/RageLog.h"
#include "RageSurface_Load.h"
#define STB_IMAGE_IMPLEMENTATION
#include <stb/stb_image.h>
#include "RageUtil/Utils/RageUtil.h"
#include "RageSurface.h"

#include <set>

RageSurfaceUtils::OpenResult
RageSurface_stb_Load(const std::string& sPath,
					 RageSurface*& ret,
					 bool bHeaderOnly,
					 std::string& error)
{
	RageFile f;
	if (!f.Open(sPath)) {
		error = f.GetError();
		return RageSurfaceUtils::OPEN_FATAL_ERROR;
	}

	int x, y, n;
	const auto doot = stbi_load(f.GetPath().c_str(), &x, &y, &n, 4);
	if (doot == nullptr) {
		return RageSurfaceUtils::OPEN_FATAL_ERROR;
	}
	if (bHeaderOnly) {
		ret = CreateSurfaceFrom(x, y, 32, 0, 0, 0, 0, nullptr, x * 4);
		stbi_image_free(doot);
		return RageSurfaceUtils::OPEN_OK;
	} else {
		ret = CreateSurfaceFrom(x,
								y,
								32,
								Swap32BE(0xFF000000),
								Swap32BE(0x00FF0000),
								Swap32BE(0x0000FF00),
								Swap32BE(0x000000FF),
								doot,
								x * 4);
	}

	if (ret == nullptr) {
		stbi_image_free(doot);
		return RageSurfaceUtils::OPEN_UNKNOWN_FILE_FORMAT; // XXX
	}
	ret->stb_loadpoint = true;
	return RageSurfaceUtils::OPEN_OK;
}
static RageSurface*
TryOpenFile(const std::string& sPath,
			bool bHeaderOnly,
			std::string& error,
			const std::string& format,
			bool& bKeepTrying)
{
	RageSurface* ret = nullptr;
	RageSurfaceUtils::OpenResult result;
	result = RageSurface_stb_Load(sPath, ret, bHeaderOnly, error);

	if (result == RageSurfaceUtils::OPEN_OK) {
		ASSERT(ret != nullptr);
		return ret;
	}

	LOG->Trace("Format %s failed: %s", format.c_str(), error.c_str());
	return nullptr;
}

RageSurface*
RageSurfaceUtils::LoadFile(const std::string& sPath,
						   std::string& error,
						   bool bHeaderOnly)
{
	{
		RageFile TestOpen;
		if (!TestOpen.Open(sPath)) {
			error = TestOpen.GetError();
			return nullptr;
		}
	}

	std::set<std::string> FileTypes;
	auto const& exts = ActorUtil::GetTypeExtensionList(FT_Bitmap);
	for (const auto& ext : exts) {
		FileTypes.insert(ext);
	}

	auto format = GetExtension(sPath);
	MakeLower(format);

	auto bKeepTrying = true;

	/* If the extension matches a format, try that first. */
	if (FileTypes.find(format) != FileTypes.end()) {
		const auto ret =
		  TryOpenFile(sPath, bHeaderOnly, error, format, bKeepTrying);
		if (ret)
			return ret;
		FileTypes.erase(format);
	}

	for (auto it = FileTypes.begin(); bKeepTrying && it != FileTypes.end();
		 ++it) {
		const auto ret =
		  TryOpenFile(sPath, bHeaderOnly, error, *it, bKeepTrying);
		if (ret) {
			LOG->UserLog("Graphic file", sPath, "is really %s", it->c_str());
			return ret;
		}
	}

	return nullptr;
}
